local SADEnabler={
	force_mod_window=false,
	use_max_survivors=true,
	max_survivors=0,
	original = {
		GetMaxSurvivorsCount = GetMaxSurvivorsCount
	}
}
rawset(_G, "SADEnabler", SADEnabler)

if FirstLoad then
end

function SADEnabler:GetMaxSurvivorsCount(scenario_id)
	if (SADEnabler.use_max_survivors and SADEnabler.max_survivors ~= 0) then   
		for key, scdef in pairs(ScenarioDefs) do
			-- seems to be unnecessary at this time. Global override below will work just fine..
			--scdef.StartingCharacters = SADEnabler.max_survivors;
		end
		return SADEnabler.max_survivors
	end

	return SADEnabler.original.GetMaxSurvivorsCount(scenario_id)
end


local function OptionsApply()
	local EnableDeveloper = GetAccountStorageOptionValue("EnableDeveloper")
	local EnableCheats = GetAccountStorageOptionValue("EnableCheats")
	local AlwaysLoadMods = GetAccountStorageOptionValue("AlwaysLoadMods")
	SADEnabler.max_survivors = GetAccountStorageOptionValue("MaxSurvivorsOverride")
	
	if (AlwaysLoadMods and not AccountOptions.LoadAllMods) then
		print('Enabled Mods Force Loading - for user account.');
		AccountStorage.LoadAllMods = true
		SaveAccountStorage(1000)
	elseif (not AlwaysLoadMods and AccountStorage.LoadAllMods) then
		AccountStorage.LoadAllMods = false
		SaveAccountStorage(1000)
	end
	
	if (EnableDeveloper and not Platform.developer) then
		Platform.developer=true;
	elseif (not EnableDeveloper and Platform.developer) then
		Platform.developer=false;
	end
	if (EnableCheats and not Platform.cheats) then
		Platform.cheats=true;
	elseif (not EnableCheats and Platform.cheats) then
		Platform.cheats=false;
	end
	
end


local function AccountReady()
	print('Account and SADEnabler ready.')
	OptionsApply();
end

local function ModDefsLoaded()
	--print('ModDefsLoaded')
	
	local AlwaysLoadMods = GetAccountStorageOptionValue("AlwaysLoadMods")
	-- the idea here is pretty simple. We only force the mod menu to show when a.) user isn't forcing load-all strategy, and b.) the game developers haven't pushed out the future DLC that enables modding. 
	-- the logic in source code clearly dictates a check for a DLC called "Modding". 
	-- so if it ever becomes true at boot up (aka time of this func running), then this code "self disables" so to speak. Since a better mod manager will then be available, rendering this moot.
	if (not AlwaysLoadMods and not g_AvailableDlc["Modding"]) then 
		SADEnabler.force_mod_window=true
		g_AvailableDlc["Modding"]=true
	end
end


-- this never gets called during game boot due to gamedeveloper source that blocks the event for DLC. And/or this thread becomes in orphan. Might be the latter actually.
-- we will have to run a thread during Reload event, to monitor for completion..
local function ModsReloaded()
end


local function adjustOptions() 
	if (OptionsCategories[#OptionsCategories].id=="X") then return end;
	
	table.iappend(OptionsCategories, {
	  {
		id = "X",
		display_name = T(8565, "X"),
		caps_name = "X",
	  },
	})

	table.iappend(OptionsObject.properties, {		
		{
		  name = "Enable Developer",
		  id = "EnableDeveloper",
		  category = "X",
		  editor = "bool",
		  default = false
		},
		{
		  name = "Enable Cheats",
		  id = "EnableCheats",
		  category = "X",
		  editor = "bool",
		  default = false
		},
		{
		  name = "Load All Mods",
		  id = "AlwaysLoadMods",
		  category = "X",
		  editor = "bool",
		  default = true
		},
		{
		  name = "Max Survivors",
		  id = "MaxSurvivorsOverride",
		  category = "X",
		  editor = "choice",
		  default = "Off",
		  items = {
			{
			  value = 0,
			  text = "Default"
			},
			{
			  value = 5,
			  text = "5"
			},
			{
			  value = 6,
			  text = "6"
			},
			{
			  value = 8,
			  text = "8"
			},
			{
			  value = 10,
			  text = "10"
			}
		  }
		},
	});
end


local function DialogSetMode(dialogSelf, mode, mParam) 
	--SADEnabler:printtable(dialogSelf)
	--print('MODE('.. dialogSelf.Id .. '): '..mode)

	if (SADEnabler.force_mod_window) then
		if (dialogSelf.Id == "idModsUIDialog") then 
			if (mode=="") then 
				CloseDialog("ModManagerDialog")
				local pgm = GetDialog('PreGameMenu')
				pgm:SetMode("Main")
			end
		elseif (dialogSelf.Id == "idPreGameMenu") then
			if (mode == "Main") then 

			elseif (mode == "ModManager") then
				OpenDialog("ModManagerDialog")
			end
		end
	end

	if (mode == "Survivors") then -- Mission, Settings, Properties, Survivors
		local dlg_context = GetDialogContext(dialogSelf);
		--SADEnabler:printtable(dlg_context)
		--SADEnabler:printtable(dlg_context.game_rules)
		
		if (dlg_context.game_rules.MaxSurvivors) then
			SADEnabler.use_max_survivors=true
		else 
			-- disabled this feature.. since there's no point in putting it into game rules. Legacy thing here for posterity..
			--SADEnabler.use_max_survivors=false
		end
	end

end



-- this doesn't appear to function, despite the code seeming to be correct..
local function InjectModButton()
	--print("PreGameMenuOpen")
	
	local pgm = GetDialog('PreGameMenu')
	--SADEnabler:printtable(pgm)
	
	if (pgm.actions[#pgm.actions].ActionId == "idMods") then 
		return
	end
	
	local bottomBarHost = pgm.actions[#pgm.actions].host
	local bb_idbl_last = bottomBarHost.idButtonsList[#bottomBarHost.idButtonsList]
	--SADEnabler:printtable(bottomBarHost.idButtonsList)
	--SADEnabler:printtable(bb_idbl_last)
	
	bottomBarHost:AddAction(XAction:new({
		ActionId = "idMods",
		ActionName = "MODS",
		ActionToolbar = "bottommenu",
		ActionToolbarSplit = true,
		OnAction = function(action, host)
			OpenDialog("ModManagerDialog")
		end,
	}))
	
end
-- leaving this legacy code here for posterity. Don't see a point in doing this as a game rule, when it's already in the Options dialog as a global hook var stored at account level.
-- servers as a decent sample of implementing a custom GameRule checkbox. Noting that the value is only persisted for NEW games, as existing ones won't have the stored value.
local function addGameRules()
	--[[
	PlaceObj("GameRuleDef", {
	  description = "Set the max survivors to value assigned in Options menu",
	  display_name = "Unlock Max Survivors",
	  exclusionlist = {},
	  group = "Default",
	  id = "MaxSurvivors",
	  
	  effects = {},
	  
	  msg_reactions = {
	  
		PlaceObj("MsgReaction", {
		  Event = "OptionsApply",
		  Handler = function(self)
			
		  end
		}),
		
		PlaceObj("MsgReaction", {
		  Event = "DialogSetMode",
		  Handler = function(self, dialogSelf, mode, mParam)
			
		  end
		}),
	  }
	})
	--]]
end




local function Boot()
	print("SADEnabler is running")
	
	adjustOptions()
	--addGameRules()
end
local function Registration()
	OnMsg.ModDefsLoaded = function()
		ModDefsLoaded()
	end
	OnMsg.ModsReloaded = function()
		ModsReloaded()
	end
	OnMsg.DialogSetMode = function(dialogSelf, mode, mParam)
		DialogSetMode(dialogSelf, mode, mParam)
	end
	OnMsg.OptionsApply = function()
		OptionsApply()
	end
	OnMsg.ReloadLua = function()
		--print('ReloadLua')
		local old_onmsg = tostring(OnMsg.Autorun) -- ghetto detection approach.
		CreateRealTimeThread(function()
			Sleep(50)
			
			while Loading do
				Sleep(20)
			end
			
			Registration();
			Boot();
		end)
	end
	
	_G.GetMaxSurvivorsCount = SADEnabler.GetMaxSurvivorsCount
	--print('SurvivorCount Orig: '..tostring(GetMaxSurvivorsCount))
	--print('SurvivorCount New: '..tostring(SADEnabler.original.GetMaxSurvivorsCount))
end

local function Preload(self)
	CreateRealTimeThread(function() 
		while not g_ConsoleFENV do
			Sleep(150)
		end
		-- todo: code this file. Since it has access to a ready console, that may be of use for exposing something for easier hooking and eval scripting, in cheat engine.
		dofile(self.folder.."/console.lua", g_ConsoleFENV._G)
	end)
	CreateRealTimeThread(function()
		WaitMsg("AccountStorageChanged")
		while not AccountStorage do
			Sleep(50)
		end
		
		AccountReady();		
	end)
end

local function Postload(self) 
	g_AvailableDlc[self.name] = true
    --g_AccessibleDlc[self.name] = true
	
	Registration()
	Boot()
end



-- some helper funcs to expose globally...

function SADEnabler:printtable(table, indent)
  if (table == nil) then
	print('nil')
	return
  end
  print(tostring(table) .. '\n')
  for index, value in pairs(table) do 
    local temp = value
	if (value==nil) then
		temp = 'nil'
	--elseif (type(value) == 'table') then
	--	printtable(value)
	end
    print('    ' .. tostring(index) .. ' : ' .. tostring(temp) .. '\n')
  end
end
function SADEnabler:shallowcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end
function SADEnabler:min(x, m)
    if not x or x < m then return m end
	return x
end
function SADEnabler:clamp(x, min, max)
    if x < min then return min end
    if x > max then return max end
    return x
end

-- todo: consider using a non-mod strategy, allowing users to simply drop LUA files into a target folder, and thus simplifying the footprint of work effort here. 
-- debatable. If the goal is to Mod stuff, then we should not do this, so as to align with developer track and online guides for surviving mars. 
-- need to verify the "AppData" reference here is fully mounted. There is a mounting operation somewhere that likely needs called first. (see Dlc.lua and Mod.lua)
-- also look into the svndeveloper/ prefix. Seems the devs already have a dedicated folder for exactly this purpose, using the original unpacked decrypted sources in full.
--dofolder('AppData/CustomLUA/');

return {
  display_name = T(8565,"SAD Enabler"),
  enabled = true,
  name = "SADEnabler",
  pops_dlc_id = "stranded_alien_dawn_enabler",
  pre_load = function(self)
	Preload(self)
  end,
  post_load = function(self)
	Postload(self)
  end,
  ps4_gid = 0,
  ps4_label = "SADEnabler",
  ps4_trophy_group_description = false,
  required_lua_revision = 227000,
  steam_dlc_id = 1234567
}
